// SPDX-FileCopyrightText: 2019 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "dprintclass.h"

DPrintClass::DPrintClass()
{
    m_type = DESTTYPE::CLASS;
}
