/****************************************************************************
** Meta object code from reading C++ file 'usbthread.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "usbprinter/usbthread.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'usbthread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_USBThread_t {
    QByteArrayData data[13];
    char stringdata0[177];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_USBThread_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_USBThread_t qt_meta_stringdata_USBThread = {
    {
QT_MOC_LITERAL(0, 0, 9), // "USBThread"
QT_MOC_LITERAL(1, 10, 19), // "newUSBDeviceArrived"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 14), // "startGetDriver"
QT_MOC_LITERAL(4, 46, 12), // "workFinished"
QT_MOC_LITERAL(5, 59, 19), // "deviceStatusChanged"
QT_MOC_LITERAL(6, 79, 18), // "defaultPrinterName"
QT_MOC_LITERAL(7, 98, 6), // "status"
QT_MOC_LITERAL(8, 105, 20), // "addArrivedUSBPrinter"
QT_MOC_LITERAL(9, 126, 25), // "notificationActionInvoked"
QT_MOC_LITERAL(10, 152, 2), // "id"
QT_MOC_LITERAL(11, 155, 3), // "msg"
QT_MOC_LITERAL(12, 159, 17) // "addingJobFinished"

    },
    "USBThread\0newUSBDeviceArrived\0\0"
    "startGetDriver\0workFinished\0"
    "deviceStatusChanged\0defaultPrinterName\0"
    "status\0addArrivedUSBPrinter\0"
    "notificationActionInvoked\0id\0msg\0"
    "addingJobFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_USBThread[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,
       4,    0,   51,    2, 0x06 /* Public */,
       5,    2,   52,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,   57,    2, 0x08 /* Private */,
       9,    2,   58,    2, 0x08 /* Private */,
      12,    1,   63,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    6,    7,

 // slots: parameters
    QMetaType::Bool,
    QMetaType::Void, QMetaType::UInt, QMetaType::QString,   10,   11,
    QMetaType::Void, QMetaType::Int,    7,

       0        // eod
};

void USBThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<USBThread *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->newUSBDeviceArrived(); break;
        case 1: _t->startGetDriver(); break;
        case 2: _t->workFinished(); break;
        case 3: _t->deviceStatusChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: { bool _r = _t->addArrivedUSBPrinter();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->notificationActionInvoked((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->addingJobFinished((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (USBThread::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&USBThread::newUSBDeviceArrived)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (USBThread::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&USBThread::startGetDriver)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (USBThread::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&USBThread::workFinished)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (USBThread::*)(const QString & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&USBThread::deviceStatusChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject USBThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_USBThread.data,
    qt_meta_data_USBThread,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *USBThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *USBThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_USBThread.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int USBThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void USBThread::newUSBDeviceArrived()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void USBThread::startGetDriver()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void USBThread::workFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void USBThread::deviceStatusChanged(const QString & _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
