/****************************************************************************
** Meta object code from reading C++ file 'dprintersshowwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "ui/dprintersshowwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dprintersshowwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ServerSettingsWindow_t {
    QByteArrayData data[3];
    char stringdata0[31];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ServerSettingsWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ServerSettingsWindow_t qt_meta_stringdata_ServerSettingsWindow = {
    {
QT_MOC_LITERAL(0, 0, 20), // "ServerSettingsWindow"
QT_MOC_LITERAL(1, 21, 8), // "finished"
QT_MOC_LITERAL(2, 30, 0) // ""

    },
    "ServerSettingsWindow\0finished\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ServerSettingsWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void ServerSettingsWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ServerSettingsWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->finished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ServerSettingsWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerSettingsWindow::finished)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ServerSettingsWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<DMainWindow::staticMetaObject>(),
    qt_meta_stringdata_ServerSettingsWindow.data,
    qt_meta_data_ServerSettingsWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ServerSettingsWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ServerSettingsWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ServerSettingsWindow.stringdata0))
        return static_cast<void*>(this);
    return DMainWindow::qt_metacast(_clname);
}

int ServerSettingsWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void ServerSettingsWindow::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_ItemDelegate_t {
    QByteArrayData data[1];
    char stringdata0[13];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ItemDelegate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ItemDelegate_t qt_meta_stringdata_ItemDelegate = {
    {
QT_MOC_LITERAL(0, 0, 12) // "ItemDelegate"

    },
    "ItemDelegate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ItemDelegate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<QStyledItemDelegate::staticMetaObject>(),
    qt_meta_stringdata_ItemDelegate.data,
    qt_meta_data_ItemDelegate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ItemDelegate.stringdata0))
        return static_cast<void*>(this);
    return QStyledItemDelegate::qt_metacast(_clname);
}

int ItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStyledItemDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_PrinterListView_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PrinterListView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PrinterListView_t qt_meta_stringdata_PrinterListView = {
    {
QT_MOC_LITERAL(0, 0, 15) // "PrinterListView"

    },
    "PrinterListView"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PrinterListView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void PrinterListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PrinterListView::staticMetaObject = { {
    QMetaObject::SuperData::link<DListView::staticMetaObject>(),
    qt_meta_stringdata_PrinterListView.data,
    qt_meta_data_PrinterListView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PrinterListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PrinterListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PrinterListView.stringdata0))
        return static_cast<void*>(this);
    return DListView::qt_metacast(_clname);
}

int PrinterListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DListView::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_DPrintersShowWindow_t {
    QByteArrayData data[36];
    char stringdata0[552];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DPrintersShowWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DPrintersShowWindow_t qt_meta_stringdata_DPrintersShowWindow = {
    {
QT_MOC_LITERAL(0, 0, 19), // "DPrintersShowWindow"
QT_MOC_LITERAL(1, 20, 19), // "addPrinterClickSlot"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 22), // "deletePrinterClickSlot"
QT_MOC_LITERAL(4, 64, 17), // "renamePrinterSlot"
QT_MOC_LITERAL(5, 82, 14), // "QStandardItem*"
QT_MOC_LITERAL(6, 97, 5), // "pItem"
QT_MOC_LITERAL(7, 103, 22), // "setPrinterNameEditable"
QT_MOC_LITERAL(8, 126, 11), // "QModelIndex"
QT_MOC_LITERAL(9, 138, 5), // "index"
QT_MOC_LITERAL(10, 144, 21), // "printSettingClickSlot"
QT_MOC_LITERAL(11, 166, 19), // "printQueueClickSlot"
QT_MOC_LITERAL(12, 186, 18), // "printTestClickSlot"
QT_MOC_LITERAL(13, 205, 19), // "printFalutClickSlot"
QT_MOC_LITERAL(14, 225, 20), // "printSupplyClickSlot"
QT_MOC_LITERAL(15, 246, 13), // "supplyFreshed"
QT_MOC_LITERAL(16, 260, 32), // "printerListWidgetItemChangedSlot"
QT_MOC_LITERAL(17, 293, 8), // "previous"
QT_MOC_LITERAL(18, 302, 20), // "contextMenuRequested"
QT_MOC_LITERAL(19, 323, 5), // "point"
QT_MOC_LITERAL(20, 329, 24), // "listWidgetMenuActionSlot"
QT_MOC_LITERAL(21, 354, 7), // "checked"
QT_MOC_LITERAL(22, 362, 22), // "refreshPrinterListView"
QT_MOC_LITERAL(23, 385, 14), // "newPrinterName"
QT_MOC_LITERAL(24, 400, 18), // "serverSettingsSlot"
QT_MOC_LITERAL(25, 419, 10), // "closeEvent"
QT_MOC_LITERAL(26, 430, 12), // "QCloseEvent*"
QT_MOC_LITERAL(27, 443, 5), // "event"
QT_MOC_LITERAL(28, 449, 11), // "resizeEvent"
QT_MOC_LITERAL(29, 461, 13), // "QResizeEvent*"
QT_MOC_LITERAL(30, 475, 19), // "printerStateChanged"
QT_MOC_LITERAL(31, 495, 12), // "QDBusMessage"
QT_MOC_LITERAL(32, 508, 3), // "msg"
QT_MOC_LITERAL(33, 512, 19), // "deviceStatusChanged"
QT_MOC_LITERAL(34, 532, 11), // "changeEvent"
QT_MOC_LITERAL(35, 544, 7) // "QEvent*"

    },
    "DPrintersShowWindow\0addPrinterClickSlot\0"
    "\0deletePrinterClickSlot\0renamePrinterSlot\0"
    "QStandardItem*\0pItem\0setPrinterNameEditable\0"
    "QModelIndex\0index\0printSettingClickSlot\0"
    "printQueueClickSlot\0printTestClickSlot\0"
    "printFalutClickSlot\0printSupplyClickSlot\0"
    "supplyFreshed\0printerListWidgetItemChangedSlot\0"
    "previous\0contextMenuRequested\0point\0"
    "listWidgetMenuActionSlot\0checked\0"
    "refreshPrinterListView\0newPrinterName\0"
    "serverSettingsSlot\0closeEvent\0"
    "QCloseEvent*\0event\0resizeEvent\0"
    "QResizeEvent*\0printerStateChanged\0"
    "QDBusMessage\0msg\0deviceStatusChanged\0"
    "changeEvent\0QEvent*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DPrintersShowWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  114,    2, 0x08 /* Private */,
       3,    0,  115,    2, 0x08 /* Private */,
       4,    1,  116,    2, 0x08 /* Private */,
       7,    1,  119,    2, 0x08 /* Private */,
      10,    0,  122,    2, 0x08 /* Private */,
      11,    0,  123,    2, 0x08 /* Private */,
      12,    0,  124,    2, 0x08 /* Private */,
      13,    0,  125,    2, 0x08 /* Private */,
      14,    0,  126,    2, 0x08 /* Private */,
      15,    2,  127,    2, 0x08 /* Private */,
      16,    1,  132,    2, 0x08 /* Private */,
      18,    1,  135,    2, 0x08 /* Private */,
      20,    1,  138,    2, 0x08 /* Private */,
      22,    1,  141,    2, 0x08 /* Private */,
      24,    0,  144,    2, 0x08 /* Private */,
      25,    1,  145,    2, 0x08 /* Private */,
      28,    1,  148,    2, 0x08 /* Private */,
      30,    1,  151,    2, 0x08 /* Private */,
      33,    1,  154,    2, 0x08 /* Private */,
      34,    1,  157,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    2,    2,
    QMetaType::Void, 0x80000000 | 8,   17,
    QMetaType::Void, QMetaType::QPoint,   19,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, QMetaType::QString,   23,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Void, 0x80000000 | 29,   27,
    QMetaType::Void, 0x80000000 | 31,   32,
    QMetaType::Void, 0x80000000 | 31,   32,
    QMetaType::Void, 0x80000000 | 35,   27,

       0        // eod
};

void DPrintersShowWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DPrintersShowWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->addPrinterClickSlot(); break;
        case 1: _t->deletePrinterClickSlot(); break;
        case 2: _t->renamePrinterSlot((*reinterpret_cast< QStandardItem*(*)>(_a[1]))); break;
        case 3: _t->setPrinterNameEditable((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 4: _t->printSettingClickSlot(); break;
        case 5: _t->printQueueClickSlot(); break;
        case 6: _t->printTestClickSlot(); break;
        case 7: _t->printFalutClickSlot(); break;
        case 8: _t->printSupplyClickSlot(); break;
        case 9: _t->supplyFreshed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 10: _t->printerListWidgetItemChangedSlot((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 11: _t->contextMenuRequested((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 12: _t->listWidgetMenuActionSlot((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->refreshPrinterListView((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->serverSettingsSlot(); break;
        case 15: _t->closeEvent((*reinterpret_cast< QCloseEvent*(*)>(_a[1]))); break;
        case 16: _t->resizeEvent((*reinterpret_cast< QResizeEvent*(*)>(_a[1]))); break;
        case 17: _t->printerStateChanged((*reinterpret_cast< const QDBusMessage(*)>(_a[1]))); break;
        case 18: _t->deviceStatusChanged((*reinterpret_cast< const QDBusMessage(*)>(_a[1]))); break;
        case 19: _t->changeEvent((*reinterpret_cast< QEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DPrintersShowWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<DMainWindow::staticMetaObject>(),
    qt_meta_stringdata_DPrintersShowWindow.data,
    qt_meta_data_DPrintersShowWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DPrintersShowWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DPrintersShowWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DPrintersShowWindow.stringdata0))
        return static_cast<void*>(this);
    return DMainWindow::qt_metacast(_clname);
}

int DPrintersShowWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
