/****************************************************************************
** Meta object code from reading C++ file 'dpropertysetdlg.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "ui/dpropertysetdlg.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dpropertysetdlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DPropertySetDlg_t {
    QByteArrayData data[23];
    char stringdata0[497];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DPropertySetDlg_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DPropertySetDlg_t qt_meta_stringdata_DPropertySetDlg = {
    {
QT_MOC_LITERAL(0, 0, 15), // "DPropertySetDlg"
QT_MOC_LITERAL(1, 16, 21), // "pageSizeCombo_clicked"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 6), // "iIndex"
QT_MOC_LITERAL(4, 46, 24), // "paperOrientation_clicked"
QT_MOC_LITERAL(5, 71, 24), // "duplexPrintCombo_clicked"
QT_MOC_LITERAL(6, 96, 22), // "colorModeCombo_clicked"
QT_MOC_LITERAL(7, 119, 23), // "printOrderCombo_clicked"
QT_MOC_LITERAL(8, 143, 24), // "paperOriginCombo_clicked"
QT_MOC_LITERAL(9, 168, 22), // "paperTypeCombo_clicked"
QT_MOC_LITERAL(10, 191, 32), // "printStapleLocationCombo_clicked"
QT_MOC_LITERAL(11, 224, 5), // "index"
QT_MOC_LITERAL(12, 230, 26), // "printBindEdgeCombo_clicked"
QT_MOC_LITERAL(13, 257, 28), // "printResolutionCombo_clicked"
QT_MOC_LITERAL(14, 286, 22), // "outputQuanlity_clicked"
QT_MOC_LITERAL(15, 309, 24), // "installAbleCombo_clicked"
QT_MOC_LITERAL(16, 334, 20), // "generalCombo_clicked"
QT_MOC_LITERAL(17, 355, 23), // "printUriUI_EditFinished"
QT_MOC_LITERAL(18, 379, 28), // "printLocationUI_EditFinished"
QT_MOC_LITERAL(19, 408, 31), // "printDescriptionUI_EditFinished"
QT_MOC_LITERAL(20, 440, 18), // "confirmBtn_clicked"
QT_MOC_LITERAL(21, 459, 17), // "cancelBtn_clicked"
QT_MOC_LITERAL(22, 477, 19) // "preferenceClickSlot"

    },
    "DPropertySetDlg\0pageSizeCombo_clicked\0"
    "\0iIndex\0paperOrientation_clicked\0"
    "duplexPrintCombo_clicked\0"
    "colorModeCombo_clicked\0printOrderCombo_clicked\0"
    "paperOriginCombo_clicked\0"
    "paperTypeCombo_clicked\0"
    "printStapleLocationCombo_clicked\0index\0"
    "printBindEdgeCombo_clicked\0"
    "printResolutionCombo_clicked\0"
    "outputQuanlity_clicked\0installAbleCombo_clicked\0"
    "generalCombo_clicked\0printUriUI_EditFinished\0"
    "printLocationUI_EditFinished\0"
    "printDescriptionUI_EditFinished\0"
    "confirmBtn_clicked\0cancelBtn_clicked\0"
    "preferenceClickSlot"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DPropertySetDlg[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  109,    2, 0x08 /* Private */,
       4,    1,  112,    2, 0x08 /* Private */,
       5,    1,  115,    2, 0x08 /* Private */,
       6,    1,  118,    2, 0x08 /* Private */,
       7,    1,  121,    2, 0x08 /* Private */,
       8,    1,  124,    2, 0x08 /* Private */,
       9,    1,  127,    2, 0x08 /* Private */,
      10,    1,  130,    2, 0x08 /* Private */,
      12,    1,  133,    2, 0x08 /* Private */,
      13,    1,  136,    2, 0x08 /* Private */,
      14,    1,  139,    2, 0x08 /* Private */,
      15,    1,  142,    2, 0x08 /* Private */,
      16,    1,  145,    2, 0x08 /* Private */,
      17,    0,  148,    2, 0x08 /* Private */,
      18,    0,  149,    2, 0x08 /* Private */,
      19,    0,  150,    2, 0x08 /* Private */,
      20,    0,  151,    2, 0x08 /* Private */,
      21,    0,  152,    2, 0x08 /* Private */,
      22,    0,  153,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void DPropertySetDlg::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DPropertySetDlg *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pageSizeCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->paperOrientation_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->duplexPrintCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->colorModeCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->printOrderCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->paperOriginCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->paperTypeCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->printStapleLocationCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->printBindEdgeCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->printResolutionCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->outputQuanlity_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->installAbleCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->generalCombo_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->printUriUI_EditFinished(); break;
        case 14: _t->printLocationUI_EditFinished(); break;
        case 15: _t->printDescriptionUI_EditFinished(); break;
        case 16: _t->confirmBtn_clicked(); break;
        case 17: _t->cancelBtn_clicked(); break;
        case 18: _t->preferenceClickSlot(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DPropertySetDlg::staticMetaObject = { {
    QMetaObject::SuperData::link<DSettingsDialog::staticMetaObject>(),
    qt_meta_stringdata_DPropertySetDlg.data,
    qt_meta_data_DPropertySetDlg,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DPropertySetDlg::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DPropertySetDlg::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DPropertySetDlg.stringdata0))
        return static_cast<void*>(this);
    return DSettingsDialog::qt_metacast(_clname);
}

int DPropertySetDlg::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DSettingsDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
