/****************************************************************************
** Meta object code from reading C++ file 'printersearchwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "ui/printersearchwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'printersearchwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PrinterSearchWindow_t {
    QByteArrayData data[29];
    char stringdata0[391];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PrinterSearchWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PrinterSearchWindow_t qt_meta_stringdata_PrinterSearchWindow = {
    {
QT_MOC_LITERAL(0, 0, 19), // "PrinterSearchWindow"
QT_MOC_LITERAL(1, 20, 17), // "updatePrinterList"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 11), // "printerName"
QT_MOC_LITERAL(4, 51, 21), // "listWidgetClickedSlot"
QT_MOC_LITERAL(5, 73, 11), // "QModelIndex"
QT_MOC_LITERAL(6, 85, 8), // "previous"
QT_MOC_LITERAL(7, 94, 19), // "getDeviceResultSlot"
QT_MOC_LITERAL(8, 114, 2), // "id"
QT_MOC_LITERAL(9, 117, 5), // "state"
QT_MOC_LITERAL(10, 123, 27), // "getDeviceResultByManualSlot"
QT_MOC_LITERAL(11, 151, 22), // "printerListClickedSlot"
QT_MOC_LITERAL(12, 174, 5), // "index"
QT_MOC_LITERAL(13, 180, 22), // "driverAutoSearchedSlot"
QT_MOC_LITERAL(14, 203, 21), // "driverManSearchedSlot"
QT_MOC_LITERAL(15, 225, 22), // "refreshPrinterListSlot"
QT_MOC_LITERAL(16, 248, 22), // "searchPrintersByManual"
QT_MOC_LITERAL(17, 271, 18), // "lineEditURIChanged"
QT_MOC_LITERAL(18, 290, 3), // "uri"
QT_MOC_LITERAL(19, 294, 17), // "installDriverSlot"
QT_MOC_LITERAL(20, 312, 17), // "driverChangedSlot"
QT_MOC_LITERAL(21, 330, 17), // "smbInfomationSlot"
QT_MOC_LITERAL(22, 348, 4), // "int&"
QT_MOC_LITERAL(23, 353, 3), // "ret"
QT_MOC_LITERAL(24, 357, 4), // "host"
QT_MOC_LITERAL(25, 362, 8), // "QString&"
QT_MOC_LITERAL(26, 371, 5), // "group"
QT_MOC_LITERAL(27, 377, 4), // "user"
QT_MOC_LITERAL(28, 382, 8) // "password"

    },
    "PrinterSearchWindow\0updatePrinterList\0"
    "\0printerName\0listWidgetClickedSlot\0"
    "QModelIndex\0previous\0getDeviceResultSlot\0"
    "id\0state\0getDeviceResultByManualSlot\0"
    "printerListClickedSlot\0index\0"
    "driverAutoSearchedSlot\0driverManSearchedSlot\0"
    "refreshPrinterListSlot\0searchPrintersByManual\0"
    "lineEditURIChanged\0uri\0installDriverSlot\0"
    "driverChangedSlot\0smbInfomationSlot\0"
    "int&\0ret\0host\0QString&\0group\0user\0"
    "password"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PrinterSearchWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   82,    2, 0x08 /* Private */,
       7,    2,   85,    2, 0x08 /* Private */,
      10,    2,   90,    2, 0x08 /* Private */,
      11,    1,   95,    2, 0x08 /* Private */,
      13,    0,   98,    2, 0x08 /* Private */,
      14,    0,   99,    2, 0x08 /* Private */,
      15,    0,  100,    2, 0x08 /* Private */,
      16,    0,  101,    2, 0x08 /* Private */,
      17,    1,  102,    2, 0x08 /* Private */,
      19,    0,  105,    2, 0x08 /* Private */,
      20,    1,  106,    2, 0x08 /* Private */,
      21,    5,  109,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    8,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    8,    9,
    QMetaType::Void, 0x80000000 | 5,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, 0x80000000 | 22, QMetaType::QString, 0x80000000 | 25, 0x80000000 | 25, 0x80000000 | 25,   23,   24,   26,   27,   28,

       0        // eod
};

void PrinterSearchWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PrinterSearchWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updatePrinterList((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->listWidgetClickedSlot((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 2: _t->getDeviceResultSlot((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 3: _t->getDeviceResultByManualSlot((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->printerListClickedSlot((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 5: _t->driverAutoSearchedSlot(); break;
        case 6: _t->driverManSearchedSlot(); break;
        case 7: _t->refreshPrinterListSlot(); break;
        case 8: _t->searchPrintersByManual(); break;
        case 9: _t->lineEditURIChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->installDriverSlot(); break;
        case 11: _t->driverChangedSlot((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->smbInfomationSlot((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< QString(*)>(_a[3])),(*reinterpret_cast< QString(*)>(_a[4])),(*reinterpret_cast< QString(*)>(_a[5]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PrinterSearchWindow::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PrinterSearchWindow::updatePrinterList)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PrinterSearchWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<DMainWindow::staticMetaObject>(),
    qt_meta_stringdata_PrinterSearchWindow.data,
    qt_meta_data_PrinterSearchWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PrinterSearchWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PrinterSearchWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PrinterSearchWindow.stringdata0))
        return static_cast<void*>(this);
    return DMainWindow::qt_metacast(_clname);
}

int PrinterSearchWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void PrinterSearchWindow::updatePrinterList(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
