/****************************************************************************
** Meta object code from reading C++ file 'installdriverwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "ui/installdriverwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'installdriverwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_InstallDriverWindow_t {
    QByteArrayData data[20];
    char stringdata0[281];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_InstallDriverWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_InstallDriverWindow_t qt_meta_stringdata_InstallDriverWindow = {
    {
QT_MOC_LITERAL(0, 0, 19), // "InstallDriverWindow"
QT_MOC_LITERAL(1, 20, 17), // "updatePrinterList"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 22), // "tabCurrentIndexChanged"
QT_MOC_LITERAL(4, 62, 23), // "currentMakerChangedSlot"
QT_MOC_LITERAL(5, 86, 5), // "maker"
QT_MOC_LITERAL(6, 92, 23), // "currentModelChangedSlot"
QT_MOC_LITERAL(7, 116, 5), // "model"
QT_MOC_LITERAL(8, 122, 23), // "getPPDFileFromLocalSlot"
QT_MOC_LITERAL(9, 146, 17), // "installDriverSlot"
QT_MOC_LITERAL(10, 164, 16), // "searchDriverSlot"
QT_MOC_LITERAL(11, 181, 18), // "driverSearchedSlot"
QT_MOC_LITERAL(12, 200, 17), // "driverRefreshSlot"
QT_MOC_LITERAL(13, 218, 2), // "id"
QT_MOC_LITERAL(14, 221, 6), // "iState"
QT_MOC_LITERAL(15, 228, 13), // "initCompleter"
QT_MOC_LITERAL(16, 242, 11), // "QCompleter*"
QT_MOC_LITERAL(17, 254, 13), // "COMPLETERNAME"
QT_MOC_LITERAL(18, 268, 4), // "name"
QT_MOC_LITERAL(19, 273, 7) // "strList"

    },
    "InstallDriverWindow\0updatePrinterList\0"
    "\0tabCurrentIndexChanged\0currentMakerChangedSlot\0"
    "maker\0currentModelChangedSlot\0model\0"
    "getPPDFileFromLocalSlot\0installDriverSlot\0"
    "searchDriverSlot\0driverSearchedSlot\0"
    "driverRefreshSlot\0id\0iState\0initCompleter\0"
    "QCompleter*\0COMPLETERNAME\0name\0strList"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_InstallDriverWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   65,    2, 0x08 /* Private */,
       4,    1,   66,    2, 0x08 /* Private */,
       6,    1,   69,    2, 0x08 /* Private */,
       8,    0,   72,    2, 0x08 /* Private */,
       9,    0,   73,    2, 0x08 /* Private */,
      10,    0,   74,    2, 0x08 /* Private */,
      11,    0,   75,    2, 0x08 /* Private */,
      12,    2,   76,    2, 0x08 /* Private */,
      15,    2,   81,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   13,   14,
    0x80000000 | 16, 0x80000000 | 17, QMetaType::QStringList,   18,   19,

       0        // eod
};

void InstallDriverWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<InstallDriverWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updatePrinterList(); break;
        case 1: _t->tabCurrentIndexChanged(); break;
        case 2: _t->currentMakerChangedSlot((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->currentModelChangedSlot((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->getPPDFileFromLocalSlot(); break;
        case 5: _t->installDriverSlot(); break;
        case 6: _t->searchDriverSlot(); break;
        case 7: _t->driverSearchedSlot(); break;
        case 8: _t->driverRefreshSlot((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 9: { QCompleter* _r = _t->initCompleter((*reinterpret_cast< COMPLETERNAME(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QCompleter**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (InstallDriverWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InstallDriverWindow::updatePrinterList)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject InstallDriverWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<DMainWindow::staticMetaObject>(),
    qt_meta_stringdata_InstallDriverWindow.data,
    qt_meta_data_InstallDriverWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *InstallDriverWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *InstallDriverWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_InstallDriverWindow.stringdata0))
        return static_cast<void*>(this);
    return DMainWindow::qt_metacast(_clname);
}

int InstallDriverWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void InstallDriverWindow::updatePrinterList()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
