/****************************************************************************
** Meta object code from reading C++ file 'zcupsmonitor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "dbus/zcupsmonitor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'zcupsmonitor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CupsMonitor_t {
    QByteArrayData data[21];
    char stringdata0[262];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CupsMonitor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CupsMonitor_t qt_meta_stringdata_CupsMonitor = {
    {
QT_MOC_LITERAL(0, 0, 11), // "CupsMonitor"
QT_MOC_LITERAL(1, 12, 21), // "signalJobStateChanged"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 2), // "id"
QT_MOC_LITERAL(4, 38, 5), // "state"
QT_MOC_LITERAL(5, 44, 7), // "message"
QT_MOC_LITERAL(6, 52, 25), // "signalPrinterStateChanged"
QT_MOC_LITERAL(7, 78, 7), // "printer"
QT_MOC_LITERAL(8, 86, 19), // "signalPrinterDelete"
QT_MOC_LITERAL(9, 106, 16), // "signalPrinterAdd"
QT_MOC_LITERAL(10, 123, 18), // "signalShowTrayIcon"
QT_MOC_LITERAL(11, 142, 4), // "show"
QT_MOC_LITERAL(12, 147, 11), // "isJobPurged"
QT_MOC_LITERAL(13, 159, 12), // "getJobNotify"
QT_MOC_LITERAL(14, 172, 3), // "job"
QT_MOC_LITERAL(15, 176, 14), // "getStateString"
QT_MOC_LITERAL(16, 191, 6), // "iState"
QT_MOC_LITERAL(17, 198, 18), // "notificationInvoke"
QT_MOC_LITERAL(18, 217, 18), // "notificationClosed"
QT_MOC_LITERAL(19, 236, 12), // "spoolerEvent"
QT_MOC_LITERAL(20, 249, 12) // "QDBusMessage"

    },
    "CupsMonitor\0signalJobStateChanged\0\0"
    "id\0state\0message\0signalPrinterStateChanged\0"
    "printer\0signalPrinterDelete\0"
    "signalPrinterAdd\0signalShowTrayIcon\0"
    "show\0isJobPurged\0getJobNotify\0job\0"
    "getStateString\0iState\0notificationInvoke\0"
    "notificationClosed\0spoolerEvent\0"
    "QDBusMessage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CupsMonitor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,   69,    2, 0x06 /* Public */,
       6,    3,   76,    2, 0x06 /* Public */,
       8,    1,   83,    2, 0x06 /* Public */,
       9,    1,   86,    2, 0x06 /* Public */,
      10,    1,   89,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    1,   92,    2, 0x0a /* Public */,
      13,    1,   95,    2, 0x0a /* Public */,
      15,    1,   98,    2, 0x0a /* Public */,
      17,    2,  101,    2, 0x09 /* Protected */,
      18,    2,  106,    2, 0x09 /* Protected */,
      19,    1,  111,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QString,    3,    4,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::QString,    7,    4,    5,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::Bool,   11,

 // slots: parameters
    QMetaType::Bool, QMetaType::Int,    3,
    QMetaType::QString, QMetaType::QVariantMap,   14,
    QMetaType::QString, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::UInt, QMetaType::QString,    2,    2,
    QMetaType::Void, QMetaType::UInt, QMetaType::UInt,    2,    2,
    QMetaType::Void, 0x80000000 | 20,    2,

       0        // eod
};

void CupsMonitor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CupsMonitor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalJobStateChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 1: _t->signalPrinterStateChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 2: _t->signalPrinterDelete((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->signalPrinterAdd((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->signalShowTrayIcon((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: { bool _r = _t->isJobPurged((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->getJobNotify((*reinterpret_cast< const QMap<QString,QVariant>(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->getStateString((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->notificationInvoke((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 9: _t->notificationClosed((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 10: _t->spoolerEvent((*reinterpret_cast< QDBusMessage(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CupsMonitor::*)(int , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CupsMonitor::signalJobStateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CupsMonitor::*)(const QString & , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CupsMonitor::signalPrinterStateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CupsMonitor::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CupsMonitor::signalPrinterDelete)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CupsMonitor::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CupsMonitor::signalPrinterAdd)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (CupsMonitor::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CupsMonitor::signalShowTrayIcon)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CupsMonitor::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_CupsMonitor.data,
    qt_meta_data_CupsMonitor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CupsMonitor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CupsMonitor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CupsMonitor.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int CupsMonitor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void CupsMonitor::signalJobStateChanged(int _t1, int _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CupsMonitor::signalPrinterStateChanged(const QString & _t1, int _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void CupsMonitor::signalPrinterDelete(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void CupsMonitor::signalPrinterAdd(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void CupsMonitor::signalShowTrayIcon(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
