/****************************************************************************
** Meta object code from reading C++ file 'zdevicemanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../Common/vendor/zdevicemanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'zdevicemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_RefreshDevicesTask_t {
    QByteArrayData data[1];
    char stringdata0[19];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RefreshDevicesTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RefreshDevicesTask_t qt_meta_stringdata_RefreshDevicesTask = {
    {
QT_MOC_LITERAL(0, 0, 18) // "RefreshDevicesTask"

    },
    "RefreshDevicesTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RefreshDevicesTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void RefreshDevicesTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject RefreshDevicesTask::staticMetaObject = { {
    QMetaObject::SuperData::link<TaskInterface::staticMetaObject>(),
    qt_meta_stringdata_RefreshDevicesTask.data,
    qt_meta_data_RefreshDevicesTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RefreshDevicesTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RefreshDevicesTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RefreshDevicesTask.stringdata0))
        return static_cast<void*>(this);
    return TaskInterface::qt_metacast(_clname);
}

int RefreshDevicesTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TaskInterface::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_RefreshDevicesByBackendTask_t {
    QByteArrayData data[1];
    char stringdata0[28];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RefreshDevicesByBackendTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RefreshDevicesByBackendTask_t qt_meta_stringdata_RefreshDevicesByBackendTask = {
    {
QT_MOC_LITERAL(0, 0, 27) // "RefreshDevicesByBackendTask"

    },
    "RefreshDevicesByBackendTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RefreshDevicesByBackendTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void RefreshDevicesByBackendTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject RefreshDevicesByBackendTask::staticMetaObject = { {
    QMetaObject::SuperData::link<RefreshDevicesTask::staticMetaObject>(),
    qt_meta_stringdata_RefreshDevicesByBackendTask.data,
    qt_meta_data_RefreshDevicesByBackendTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RefreshDevicesByBackendTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RefreshDevicesByBackendTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RefreshDevicesByBackendTask.stringdata0))
        return static_cast<void*>(this);
    return RefreshDevicesTask::qt_metacast(_clname);
}

int RefreshDevicesByBackendTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = RefreshDevicesTask::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_RefreshDevicesByHostTask_t {
    QByteArrayData data[10];
    char stringdata0[87];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RefreshDevicesByHostTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RefreshDevicesByHostTask_t qt_meta_stringdata_RefreshDevicesByHostTask = {
    {
QT_MOC_LITERAL(0, 0, 24), // "RefreshDevicesByHostTask"
QT_MOC_LITERAL(1, 25, 17), // "signalSmbPassWord"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 4), // "int&"
QT_MOC_LITERAL(4, 49, 3), // "ret"
QT_MOC_LITERAL(5, 53, 4), // "host"
QT_MOC_LITERAL(6, 58, 8), // "QString&"
QT_MOC_LITERAL(7, 67, 5), // "group"
QT_MOC_LITERAL(8, 73, 4), // "user"
QT_MOC_LITERAL(9, 78, 8) // "password"

    },
    "RefreshDevicesByHostTask\0signalSmbPassWord\0"
    "\0int&\0ret\0host\0QString&\0group\0user\0"
    "password"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RefreshDevicesByHostTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    5,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString, 0x80000000 | 6, 0x80000000 | 6, 0x80000000 | 6,    4,    5,    7,    8,    9,

       0        // eod
};

void RefreshDevicesByHostTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RefreshDevicesByHostTask *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalSmbPassWord((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< QString(*)>(_a[3])),(*reinterpret_cast< QString(*)>(_a[4])),(*reinterpret_cast< QString(*)>(_a[5]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RefreshDevicesByHostTask::*)(int & , QString , QString & , QString & , QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RefreshDevicesByHostTask::signalSmbPassWord)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject RefreshDevicesByHostTask::staticMetaObject = { {
    QMetaObject::SuperData::link<RefreshDevicesTask::staticMetaObject>(),
    qt_meta_stringdata_RefreshDevicesByHostTask.data,
    qt_meta_data_RefreshDevicesByHostTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RefreshDevicesByHostTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RefreshDevicesByHostTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RefreshDevicesByHostTask.stringdata0))
        return static_cast<void*>(this);
    return RefreshDevicesTask::qt_metacast(_clname);
}

int RefreshDevicesByHostTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = RefreshDevicesTask::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void RefreshDevicesByHostTask::signalSmbPassWord(int & _t1, QString _t2, QString & _t3, QString & _t4, QString & _t5)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
