/****************************************************************************
** Meta object code from reading C++ file 'helperinterface.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "dbus/helperinterface.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'helperinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_HelperInterface_t {
    QByteArrayData data[22];
    char stringdata0[297];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HelperInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HelperInterface_t qt_meta_stringdata_HelperInterface = {
    {
QT_MOC_LITERAL(0, 0, 15), // "HelperInterface"
QT_MOC_LITERAL(1, 16, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 32, 23), // "com.deepin.print.helper"
QT_MOC_LITERAL(3, 56, 21), // "signalJobStateChanged"
QT_MOC_LITERAL(4, 78, 0), // ""
QT_MOC_LITERAL(5, 79, 2), // "id"
QT_MOC_LITERAL(6, 82, 5), // "state"
QT_MOC_LITERAL(7, 88, 7), // "message"
QT_MOC_LITERAL(8, 96, 25), // "signalPrinterStateChanged"
QT_MOC_LITERAL(9, 122, 7), // "printer"
QT_MOC_LITERAL(10, 130, 19), // "signalPrinterDelete"
QT_MOC_LITERAL(11, 150, 16), // "signalPrinterAdd"
QT_MOC_LITERAL(12, 167, 19), // "deviceStatusChanged"
QT_MOC_LITERAL(13, 187, 18), // "defaultPrinterName"
QT_MOC_LITERAL(14, 206, 6), // "status"
QT_MOC_LITERAL(15, 213, 16), // "usbDeviceProcess"
QT_MOC_LITERAL(16, 230, 11), // "isJobPurged"
QT_MOC_LITERAL(17, 242, 12), // "getJobNotify"
QT_MOC_LITERAL(18, 255, 3), // "job"
QT_MOC_LITERAL(19, 259, 14), // "getStateString"
QT_MOC_LITERAL(20, 274, 6), // "iState"
QT_MOC_LITERAL(21, 281, 15) // "setTypeAndState"

    },
    "HelperInterface\0D-Bus Interface\0"
    "com.deepin.print.helper\0signalJobStateChanged\0"
    "\0id\0state\0message\0signalPrinterStateChanged\0"
    "printer\0signalPrinterDelete\0"
    "signalPrinterAdd\0deviceStatusChanged\0"
    "defaultPrinterName\0status\0usbDeviceProcess\0"
    "isJobPurged\0getJobNotify\0job\0"
    "getStateString\0iState\0setTypeAndState"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HelperInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      10,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    3,   66,    4, 0x06 /* Public */,
       8,    3,   73,    4, 0x06 /* Public */,
      10,    1,   80,    4, 0x06 /* Public */,
      11,    1,   83,    4, 0x06 /* Public */,
      12,    2,   86,    4, 0x06 /* Public */,
      15,    0,   91,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    1,   92,    4, 0x0a /* Public */,
      17,    1,   95,    4, 0x0a /* Public */,
      19,    1,   98,    4, 0x0a /* Public */,
      21,    1,  101,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QString,    5,    6,    7,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::QString,    9,    6,    7,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,   13,   14,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Bool, QMetaType::Int,    5,
    QMetaType::QString, QMetaType::QVariantMap,   18,
    QMetaType::QString, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   14,

       0        // eod
};

void HelperInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HelperInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalJobStateChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 1: _t->signalPrinterStateChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 2: _t->signalPrinterDelete((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->signalPrinterAdd((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->deviceStatusChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->usbDeviceProcess(); break;
        case 6: { bool _r = _t->isJobPurged((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->getJobNotify((*reinterpret_cast< const QMap<QString,QVariant>(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: { QString _r = _t->getStateString((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setTypeAndState((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HelperInterface::*)(int , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HelperInterface::signalJobStateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HelperInterface::*)(const QString & , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HelperInterface::signalPrinterStateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (HelperInterface::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HelperInterface::signalPrinterDelete)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (HelperInterface::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HelperInterface::signalPrinterAdd)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (HelperInterface::*)(const QString & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HelperInterface::deviceStatusChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (HelperInterface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HelperInterface::usbDeviceProcess)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject HelperInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_HelperInterface.data,
    qt_meta_data_HelperInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HelperInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HelperInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HelperInterface.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int HelperInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void HelperInterface::signalJobStateChanged(int _t1, int _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void HelperInterface::signalPrinterStateChanged(const QString & _t1, int _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void HelperInterface::signalPrinterDelete(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void HelperInterface::signalPrinterAdd(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void HelperInterface::deviceStatusChanged(const QString & _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void HelperInterface::usbDeviceProcess()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
